# -*- coding: utf-8 -*-

"""Includes functions for manipulating strings."""

import os

from nxstart.version import __version__ as version

TITLE_TEXT = """                                                 
#    # #    #        ####  #####   ##   #####  #####    
##   #  #  #        #        #    #  #  #    #   #      
# #  #   ##   #####  ####    #   #    # #    #   #      
#  # #   ##              #   #   ###### #####    #      
#   ##  #  #        #    #   #   #    # #   #    #      
#    # #    #        ####    #   #    # #    #   #                                                          
    """

VERSION_STRING = "v%s - by roedesh <Ruud Schroën>" % version


def generate_folder_name_and_path(name, cwd):
    """
    Returns a folder name and path based on the project name and current working directory.

    :param name: Project name
    :param cwd: Current working directory
    :return: New folder name and path
    """
    folder_name = name.lower().replace(" ", "_")
    folder_path = os.path.join(cwd, folder_name)
    return folder_name, folder_path



# -*- coding: utf-8 -*-

"""Includes test for the 'pynx' command"""

from click.testing import CliRunner

from nxstart.cli import cli
from nxstart.tests.helpers import (APP_AUTHOR, APP_NAME, DATE_CREATED,
                                   DIRECTORY_NAME, directory_exists,
                                   file_contains_strings,
                                   readme_has_project_and_author_name)


def test_pynx():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ["-n", APP_NAME, "-a", APP_AUTHOR, "pynx"])
        assert not result.exception
        assert result.output.endswith("Successfully created the PyNX project!\n")
        assert directory_exists(DIRECTORY_NAME)
        assert file_contains_strings("main.py", [APP_NAME, APP_AUTHOR, DATE_CREATED])
        assert readme_has_project_and_author_name()



# -*- coding: utf-8 -*-

"""Contains helper functions to use in tests"""

import datetime
import os

# Some constants to be used in tests
APP_NAME = "Test project"
APP_AUTHOR = "Ruud Schroën"
DATE_CREATED = datetime.datetime.now().strftime("%Y-%m-%d")
DIRECTORY_NAME = "test_project"


def directory_exists(folder_name=DIRECTORY_NAME):
    """
    Check if a folder exists
    """
    return os.path.isdir(folder_name)


def file_exists(file_path):
    """
    Check if the file at file_path exists.

    :param file_path: File to check
    """
    return os.path.isfile(os.path.join(DIRECTORY_NAME, file_path))


def file_contains_strings(file_path, strings):
    """
    Make sure that each given string is in the file.

    :param file_path: File to check
    :param strings: List of strings
    """
    with open(os.path.join(DIRECTORY_NAME, file_path), "r") as file:
        data = file.read().replace("\n", "")
        for s in strings:
            if s not in data:
                return False
    return True


def readme_has_project_and_author_name():
    """
    Checks that the README.md contains the project and author name.
    """
    return file_contains_strings("README.md", [APP_NAME, APP_AUTHOR])


def makefile_has_project_and_author_name():
    """
    Check that the Makefile contains the project and author name.
    """
    return file_contains_strings("Makefile", [APP_NAME, APP_AUTHOR])



# -*- coding: utf-8 -*-

"""Includes tests for the 'libt' command"""

from click.testing import CliRunner

from nxstart.cli import cli
from nxstart.tests.helpers import (APP_AUTHOR, APP_NAME, DATE_CREATED,
                                   DIRECTORY_NAME, directory_exists,
                                   file_contains_strings, file_exists,
                                   makefile_has_project_and_author_name,
                                   readme_has_project_and_author_name)


def test_libt_with_clion():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(
            cli, ["-n", APP_NAME, "-a", APP_AUTHOR, "libt", "--clion"]
        )
        assert not result.exception
        assert result.output.endswith(
            "Successfully created the libtransistor project!\n"
        )
        assert directory_exists(DIRECTORY_NAME)
        assert file_exists("CMakeLists.txt")
        assert readme_has_project_and_author_name()
        assert makefile_has_project_and_author_name()
        assert main_c_has_valid_data()


def test_libt_without_clion():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(
            cli, ["-n", APP_NAME, "-a", APP_AUTHOR, "libt", "--no-clion"]
        )
        assert not result.exception
        assert result.output.endswith(
            "Successfully created the libtransistor project!\n"
        )
        assert directory_exists(DIRECTORY_NAME)
        assert not file_exists("CMakeLists.txt")
        assert readme_has_project_and_author_name()
        assert makefile_has_project_and_author_name()
        assert main_c_has_valid_data()


def main_c_has_valid_data():
    return file_contains_strings("main.c", [APP_NAME, APP_AUTHOR, DATE_CREATED])



# -*- coding: utf-8 -*-

"""Defines the app context and commands."""

import os

import click

from nxstart import app
from nxstart.utils.strings import TITLE_TEXT, VERSION_STRING
from nxstart.version import __version__ as version


class Context(object):
    """
    Context to be passed to the sub-commands.
    """

    def __init__(self):
        self.name = None
        self.author = None
        self.cwd = os.getcwd()


pass_context = click.make_pass_decorator(Context, ensure=True)


@click.group()
@click.option("--name", "-n", default=None, help="The name of your project")
@click.option("--author", "-a", default=None, help="The full name of the author")
@pass_context
def cli(ctx, name, author):
    click.echo(TITLE_TEXT)
    click.echo(VERSION_STRING)
    if not name:
        name = click.prompt("Please enter the name of your project", type=str)
    if not author:
        author = click.prompt("Please enter your name", type=str)
    ctx.name = name
    ctx.author = author


@cli.command("libnx", short_help="create a new libnx project (C++)")
@click.option(
    "--clion/--no-clion",
    default=False,
    prompt="Are you using CLion?",
    help="include CMakeLists.txt",
)
@pass_context
def libnx(ctx, clion):
    """
    Command for generating a libnx project.

    :param ctx: Context
    :param clion: Using CLion
    """
    app.libnx(ctx.name, ctx.author, clion, ctx.cwd)


@cli.command("libt", short_help="create a new libtransistor project (C)")
@click.option(
    "--clion/--no-clion",
    default=False,
    prompt="Are you using CLion?",
    help="include CMakeLists.txt",
)
@pass_context
def libt(ctx, clion):
    """
    Command for generating a libtransistor project.

    :param ctx: Context
    :param clion: Using CLion
    """
    app.libt(ctx.name, ctx.author, clion, ctx.cwd)


@cli.command("brewjs", short_help="create a new BrewJS project (Javascript)")
@pass_context
def brewjs(ctx):
    """
    Command for generating a BrewJS project.

    :param ctx: Context
    """
    app.brewjs(ctx.name, ctx.author, ctx.cwd)


@cli.command("pynx", short_help="create a new PyNX project (Python)")
@pass_context
def pynx(ctx):
    """
    Command for generating a PyNX project.

    :param ctx: Context
    """
    app.pynx(ctx.name, ctx.author, ctx.cwd)



# -*- coding: utf-8 -*-

"""Includes tests for the 'libnx' command"""

import os

from click.testing import CliRunner

from nxstart.cli import cli
from nxstart.tests.helpers import (APP_AUTHOR, APP_NAME, DATE_CREATED,
                                   DIRECTORY_NAME, directory_exists,
                                   file_contains_strings, file_exists,
                                   makefile_has_project_and_author_name,
                                   readme_has_project_and_author_name)


def test_libnx_with_clion():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(
            cli, ["-n", APP_NAME, "-a", APP_AUTHOR, "libnx", "--clion"]
        )
        assert not result.exception
        assert result.output.endswith("Successfully created the libnx project!\n")
        assert directory_exists(os.path.join(DIRECTORY_NAME, "data"))
        assert directory_exists(os.path.join(DIRECTORY_NAME, "includes"))
        assert file_exists("CMakeLists.txt")
        assert readme_has_project_and_author_name()
        assert makefile_has_project_and_author_name()
        assert main_cpp_has_valid_data()


def test_libnx_without_clion():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(
            cli, ["-n", APP_NAME, "-a", APP_AUTHOR, "libnx", "--no-clion"]
        )
        assert not result.exception
        assert result.output.endswith("Successfully created the libnx project!\n")
        assert directory_exists(os.path.join(DIRECTORY_NAME, "data"))
        assert directory_exists(os.path.join(DIRECTORY_NAME, "includes"))
        assert not file_exists("CMakeLists.txt")
        assert readme_has_project_and_author_name()
        assert makefile_has_project_and_author_name()
        assert main_cpp_has_valid_data()


def main_cpp_has_valid_data():
    return file_contains_strings(
        os.path.join("source", "main.cpp"), [APP_NAME, APP_AUTHOR, DATE_CREATED]
    )



# -*- coding: utf-8 -*-

"""Includes functions for copying the libnx template files."""

import datetime
import os
from distutils.dir_util import copy_tree

from nxstart.utils.files import (check_and_create_directory, get_full_path,
                                 replace_in_file)


def create_libnx_project(folder_path, name, author):
    """
    Copies the files from templates/base to folder_path and modifies Makefile and source/main.cpp
    to include the project name, author name and current date.

    :param folder_path: Path to copy the files to
    :param name: Name of the project
    :param author: Name of the author
    """
    template_folder = get_full_path(os.path.join("templates", "libnx"))
    copy_tree(template_folder, folder_path)

    main_cpp_file = os.path.join(folder_path, "source", "main.cpp")
    main_cpp_replacements = {
        "APP_AUTHOR_PLACEHOLDER": author,
        "APP_NAME_PLACEHOLDER": name,
        "DATE_PLACEHOLDER": datetime.datetime.now().strftime("%Y-%m-%d"),
    }
    replace_in_file(main_cpp_file, main_cpp_replacements)

    makefile = os.path.join(folder_path, "Makefile")
    makefile_replacements = {
        "APP_NAME_PLACEHOLDER": name,
        "APP_AUTHOR_PLACEHOLDER": author,
    }
    replace_in_file(makefile, makefile_replacements)

    check_and_create_directory(os.path.join(folder_path, "data"))
    check_and_create_directory(os.path.join(folder_path, "includes"))



# -*- coding: utf-8 -*-

"""Includes test for the 'brewjs' command"""

import os

from click.testing import CliRunner

from nxstart.cli import cli
from nxstart.tests.helpers import (APP_AUTHOR, APP_NAME, DATE_CREATED,
                                   DIRECTORY_NAME, directory_exists,
                                   file_contains_strings)


def test_brewjs():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ["-n", APP_NAME, "-a", APP_AUTHOR, "brewjs"])
        assert not result.exception
        assert result.output.endswith("Successfully created the BrewJS project!\n")
        assert directory_exists(os.path.join(DIRECTORY_NAME, "assets"))
        assert file_contains_strings("Source.js", [APP_NAME, APP_AUTHOR, DATE_CREATED])
        assert file_contains_strings("package.json", [APP_NAME, APP_AUTHOR])



# -*- coding: utf-8 -*-

"""Includes functions for copying the BrewJS template files."""

import datetime
import os
from distutils.dir_util import copy_tree

from nxstart.utils.files import (check_and_create_directory, get_full_path,
                                 replace_in_file)


def create_brewjs_project(folder_path, name, author):
    """
    Copies the files from templates/js to folder_path and modifies Source.js and package.json to include the app and
    author name.

    :param folder_path: Path to copy the files to
    :param name: Name of the project
    :param author: Name of the author
    """
    template_folder = get_full_path(os.path.join("templates", "brewjs"))
    copy_tree(template_folder, folder_path)

    main_js_file = os.path.join(folder_path, "Source.js")
    main_js_replacements = {
        "APP_AUTHOR_PLACEHOLDER": author,
        "APP_NAME_PLACEHOLDER": name,
        "DATE_PLACEHOLDER": datetime.datetime.now().strftime("%Y-%m-%d"),
    }
    replace_in_file(main_js_file, main_js_replacements)

    package_json_file = os.path.join(folder_path, "package.json")
    package_json_replacements = {
        "APP_AUTHOR_PLACEHOLDER": author,
        "APP_NAME_PLACEHOLDER": name,
        "DATE_PLACEHOLDER": datetime.datetime.now().strftime("%Y-%m-%d"),
    }
    replace_in_file(package_json_file, package_json_replacements)

    check_and_create_directory(os.path.join(folder_path, "assets"))



# -*- coding: utf-8 -*-

"""Includes tests for the functions in nxstart.utils.files"""

import os

from nxstart.tests.helpers import DIRECTORY_NAME, file_contains_strings
from nxstart.utils.files import (PROJECT_ROOT, check_and_create_directory,
                                 get_full_path, replace_in_file)


def test_get_full_path():
    path = os.path.join(PROJECT_ROOT, "test.py")
    path_two = get_full_path("test.py")
    assert path == path_two


def test_check_and_create_directory(tmpdir):
    new_folder_path = str(tmpdir.join(DIRECTORY_NAME))

    check_and_create_directory(new_folder_path)
    assert os.path.isdir(new_folder_path)


def test_replace_in_file(tmpdir):
    new_test_file_path = str(tmpdir.join("testfile.txt"))
    with open(new_test_file_path, "w") as f:
        f.write("TEXT_PLACEHOLDER")
    replace_in_file(new_test_file_path, {"TEXT_PLACEHOLDER": "NEW_TEXT"})
    assert file_contains_strings(new_test_file_path, ["NEW_TEXT"])



from setuptools import setup, find_packages

import os

dir_path = os.path.dirname(os.path.realpath(__file__))

# Get the version number
version = {}
with open(os.path.join(dir_path, "nxstart", "version.py")) as fp:
    exec(fp.read(), version)


def readme():
    with open("README.rst") as f:
        return f.read()


setup(
    name="nxstart",
    description="Nintendo Switch homebrew project generator",
    long_description=readme(),
    long_description_content_type="text/markdown",
    version=version["__version__"],
    url="https://github.com/roedesh/nxstart",
    author="Ruud Schroën",
    license="MIT",
    keywords="nintendo switch libnx nx homebrew project generator",
    packages=find_packages(),
    include_package_data=True,
    install_requires=["Click"],
    entry_points="""
        [console_scripts]
        nxstart=nxstart.cli:cli
    """,
    project_urls={
        "Bug Reports": "https://github.com/roedesh/nxstart/issues",
        "Source": "https://github.com/roedesh/nxstart",
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Code Generators",
        "Topic :: Utilities",
    ],
)
